class IdleGame{
	constructor() {
		this._level = new GameLevel();
		this._level.start();
	}

	oneTurn(){
		this._level.hitAllMonsters();
	}

	isRunning(){
		return !this._level.allMonstersPassedOut();
	}

	toString(){
		return this._level.toString();
	}
}

class GameLevel{
	constructor() {
		this._monsters = [];
	}

	start(){
		this._monsters = [];
		this._monsters.push( new LargeMonster() );
		this._monsters.push( new LargeMonster() );
		this._monsters.push( new FurryMonster() );
		this._monsters.push( new FurryMonster() );
		this._monsters.push( new InvisibleMonster() );
		this._monsters.push( new InvisibleMonster() );
	}

	toString(){
		return this._monsters.join(' / ');
	}

	allMonstersPassedOut(){
		let allPassedOut = true; // assume they all have passed out!
		for (let m of this._monsters){
			if (!m.hasPassedOut()){
				allPassedOut = false; // realize: at least 1 is still awake!
				break;
			}
		}
		return allPassedOut;
	}

	hitAllMonsters(){
		for (let m of this._monsters){
			if (!m.hasPassedOut()){ // only hitting monsters who are awake!
				let damage = Math.floor(Math.random()*6+1); // roll 1 dice
				m.hit(damage);
			}
		}
	}

}

/* abstract ??? */ 
class Monster{
	constructor(){
		this._hp = 80; // initial HP value
	}

	// default behavior for any monster
	hit(damagePoints){
		this._hp -= damagePoints;
	}

	hasPassedOut(){
		if (this._hp<=0)
			return true;
		return false;
	}

	toString(){
		return `M:${this._hp}`;
	}
}

class LargeMonster extends Monster {}
class FurryMonster extends Monster {}
class InvisibleMonster extends Monster {}

// ****************** main program ******************

let g = new IdleGame();
let currentTurn = 0;
console.log(currentTurn +' -> '+ g.toString());
while (g.isRunning()){
	currentTurn++;
	g.oneTurn();
	console.log(currentTurn +' -> '+ g.toString());
}